from __future__ import absolute_import, unicode_literals

from django import forms
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.urlresolvers import NoReverseMatch, reverse
from django.db import models, transaction
from django.forms.formsets import all_valid
from django.forms.models import modelform_factory, inlineformset_factory
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import redirect, render
from django.utils import six
from django.utils.encoding import force_text
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _, ugettext

from towel import deletion, paginator
from towel.forms import towel_formfield_callback
from towel.utils import (
    app_model_label, related_classes, safe_queryset_and, tryreverse)


class ModelView(object):
    """
    ``ModelView`` offers list views, detail views and CRUD functionality
    """

    # Every view is wrapped with this decorator. Use this if you need
    # f.e. a simple way of ensuring a user is logged in before accessing
    # any view here.
    def view_decorator(self, func):
        """
        Returns the function as-is, does not apply permission checking or
        anything.
        """
        return func

    def crud_view_decorator(self, func):
        """
        Does what ``view_decorator`` above does (which is returning the passed
        view without applying any permission checking or anything).
        """
        return self.view_decorator(func)

    #: Used for detail and edit views
    template_object_name = 'object'

    #: Used for list views
    template_object_list_name = 'object_list'

    #: The base template which all default modelview templates inherit
    #: from
    base_template = 'base.html'

    #: The regular expression for detail URLs. Override this if you
    #: do not want the primary key in the URL.
    urlconf_detail_re = r'(?P<pk>\d+)'

    #: Paginate list views by this much. ``None`` means no pagination (the
    #: default).
    paginate_by = None

    #: By default, showing all objects on one page is allowed
    pagination_all_allowed = True

    #: The paginator class used for pagination
    paginator_class = paginator.Paginator

    #: The editing form class
    form_class = None

    #: Inline formsets. The key is the formset prefix and the formsets dict
    #: key at the same time, the value should be a dict instance which is
    #: used for the inlineformset_factory invocation.
    #:
    #: Example::
    #:
    #:     inlineformset_config = {
    #:         'steps': {'model': Step, 'form': CustomizedStepForm},
    #:         }
    #:
    inlineformset_config = {}

    #: Search form class
    search_form = None

    #: Search form is not only shown on list pages
    search_form_everywhere = False

    #: The form used for batch processing
    batch_form = None

    #: Messages dictionary to centrally control all possible messages
    default_messages = {
        'object_created': (
            messages.SUCCESS,
            _('The new object has been successfully created.')),
        'adding_denied': (
            messages.ERROR,
            _('You are not allowed to add objects.')),
        'object_updated': (
            messages.SUCCESS,
            _('The object has been successfully updated.')),
        'editing_denied': (
            messages.ERROR,
            _('You are not allowed to edit this object.')),
        'object_deleted': (
            messages.SUCCESS,
            _('The object has been successfully deleted.')),
        'deletion_denied': (
            messages.ERROR,
            _('You are not allowed to delete this object.')),
        'deletion_denied_related': (
            messages.ERROR,
            _('Deletion not allowed: There are %(pretty_classes)s related'
                ' to this object.')),
    }

    #: User defined messages
    custom_messages = {}

    def add_message(self, request, message, variables=None, level=None,
                    ignore=None, **kwargs):
        """
        This helper function is used to easily add messages for the current
        user.

        ``message`` may either be a key for the ``custom_messages`` and
        ``default_messages`` dictionary or a string containing the message
        itself.

        ``variables`` may be a dictionary used for string interpolation if the
        message contains placeholders.

        Some default messages already contain a message level specification
        (currently either ``SUCCESS`` or ``ERROR``). The default for all other
        messages is ``messages.INFO``, but this can be overridden by setting
        the ``level`` argument.

        If ``ignore`` is set to a list of message keys, all matching
        ``add_message`` calls during the current request-response cycle will
        be silently ignored.

        Additional keyword arguments are passed on directly to
        ``messages.add_message``. This can be used f.e. to easily add extra
        tags to the message.

        Usage::

            # Add the default object_created message
            self.add_message(request, 'object_created')

            # Add a custom message with a custom level
            self.add_message(request, _('Whatever you mean'),
                level=messages.WARNING)

            # Add extra tags to a default message
            self.add_message(request, 'deletion_denied', extra_tags='funky')

            # Fail loudly, please, if messages aren't enabled
            self.add_message(request, 'editing_denied', fail_silently=False)
        """

        message = force_text(message)

        ignorable = getattr(request, '_towel_add_message_ignore', [])
        if message in ignorable:
            return
        if ignore is not None:
            request._towel_add_message_ignore = ignore

        if message in self.custom_messages:
            message = self.custom_messages[message]
        elif message in self.default_messages:
            message = self.default_messages[message]

        if not isinstance(message, six.string_types):
            level = message[0]
            message = message[1]

        if variables:
            message = message % variables

        if level is None:
            level = messages.INFO

        # We should not fail if the messages framework is disabled
        kwargs.setdefault('fail_silently', True)

        messages.add_message(request, level, message, **kwargs)

    def __init__(self, model, **kwargs):
        self.model = model
        for key, value in kwargs.items():
            if not hasattr(self, key):
                raise TypeError('%s() received an invalid keyword %r' % (
                    self.__class__.__name__, key))
            setattr(self, key, value)

        if not hasattr(self.model, 'get_absolute_url'):
            # Add a simple primary key based URL to the model if it does not
            # have one yet
            self.model.get_absolute_url = models.permalink(
                lambda self: (
                    '%s_%s_detail' % app_model_label(self),
                    (self.pk,),
                    {}))

    def get_query_set(self, request, *args, **kwargs):
        """
        The queryset returned here is used for everything. Override this
        method if you want to show only a subset of objects to the current
        visitor.
        """
        return self.model._default_manager.all()

    def get_template(self, request, action):
        """
        Construct and return a template name for the given action.

        Example::

            self.get_template(request, 'list')

        returns the following template names for ``auth.User``::

            [
                'auth/user_list.html',
                'modelview/object_list.html',
            ]
        """
        return [
            '%s/%s_%s.html' % (app_model_label(self.model) + (action,)),
            'modelview/object_%s.html' % action,
        ]

    def get_urls(self):
        """
        Return all URLs known to this ``ModelView``. You probably do not
        need to override this except if you need more granular view
        decoration than offered with ``view_decorator`` and
        ``crud_view_decorator``. If you need additional URLs, use
        ``additional_urls`` instead.
        """
        from django.conf.urls import url
        info = app_model_label(self.model)

        urlpatterns = [
            url(r'^$',
                self.view_decorator(self.list_view),
                name='%s_%s_list' % info),
            url(r'^add/$',
                self.crud_view_decorator(self.add_view),
                name='%s_%s_add' % info),
            url(r'^%s/edit/$' % self.urlconf_detail_re,
                self.crud_view_decorator(self.edit_view),
                name='%s_%s_edit' % info),
            url(r'^%s/delete/$' % self.urlconf_detail_re,
                self.crud_view_decorator(self.delete_view),
                name='%s_%s_delete' % info),
        ]

        for spec in self.additional_urls():
            urlp, view = spec[:2]
            if len(spec) > 2:
                ident = spec[2]
            else:
                ident = view.__name__

            urlpatterns.extend([
                url(
                    urlp % {
                        'detail': self.urlconf_detail_re,
                        'ident': ident,
                    },
                    view,
                    name=('%s_%s_%%s' % info) % ident
                ),
            ])

        urlpatterns.append(
            url(r'^%s/$' % self.urlconf_detail_re,
                self.view_decorator(self.detail_view),
                name='%s_%s_detail' % info),
        )

        return urlpatterns

    def additional_urls(self):
        """
        Define additional URLs for the modelview.

        You are responsible yourself for wrapping the views with permission
        decorators etc.

        The following example will add three views; %(detail)s is replaced with
        the regular expression ``urlconf_detail_re``. The URL pattern name is
        determined by ``<app_label>_<model_name>_<ident>``, where ``ident`` is
        either the third argument in the URL specification or the function name
        of the passed view::

            return [
                (r'^autocomplete/$',
                    self.view_decorator(self.autocomplete)),
                (r'^%(detail)s/statistics/$',
                    self.view_decorator(self.statistics), 'anything'),
                (r'^%(detail)s/something/$',
                    self.crud_view_decorator(self.something), 'something'),
                ]
        """

        return ()

    @property
    def urls(self):
        """
        Property returning the return value of ``get_urls``. Should
        be used inside the URLconf::

            from towel.modelview import ModelView

            urlpatterns = [
                url(r'^prefix/', include(ModelView(Model).urls)),
            ]
        """
        return self.get_urls()

    # HELPERS

    def get_object(self, request, *args, **kwargs):
        """
        Return an instance, raising ``DoesNotExist`` if an error occurs.

        The default implementation simply passes ``*args`` and
        ``**kwargs`` into ``queryset.get``.
        """
        queryset = self.get_query_set(request, *args, **kwargs)

        try:
            return queryset.get(*args, **kwargs)
        except (ValueError, ValidationError):
            raise self.model.DoesNotExist('No %s matches the given query.' % (
                self.model._meta.object_name))

    def get_object_or_404(self, request, *args, **kwargs):
        """
        Return an instance, raising a 404 if it could not be found.
        """
        try:
            return self.get_object(request, *args, **kwargs)
        except self.model.DoesNotExist:
            raise Http404('No %s matches the given query.' % (
                self.model._meta.object_name))

    def get_formfield_callback(self, request):
        """
        Returns a formfield callback used to initialize the model form and
        inline formsets.
        """
        return towel_formfield_callback

    def get_form(self, request, instance=None, change=None, **kwargs):
        """
        Return a form class for further use by add and edit views.

        Override this if you want to specify your own form class used
        for creating and editing objects.
        """

        kwargs.setdefault(
            'formfield_callback', self.get_formfield_callback(request))
        kwargs.setdefault('form', self.form_class or forms.ModelForm)
        kwargs.setdefault('exclude', ())

        return modelform_factory(self.model, **kwargs)

    def extend_args_if_post(self, request, args):
        """
        Helper which prepends POST and FILES to args if request method
        was POST. Ugly and helpful::

            args = self.extend_args_if_post(request, [])
            form = Form(*args, **kwargs)
        """

        if request.method == 'POST':
            args[:0] = [request.POST, request.FILES]

        return args

    def get_form_instance(self, request, form_class, instance=None,
                          change=None, **kwargs):
        """
        Returns the form instance

        Override this if your form class has special needs for instantiation.
        """
        args = self.extend_args_if_post(request, [])
        kwargs['instance'] = instance

        return form_class(*args, **kwargs)

    def get_formset_instances(self, request, instance=None, change=None,
                              **kwargs):
        """
        Return a dict of formset instances. You may freely choose the
        keys for this dict, use a SortedDict or something else as long
        as it has a 'values()' method.

        Please note that the instance passed here has not necessarily
        been saved to the database yet.
        """
        args = self.extend_args_if_post(request, [])
        kwargs['instance'] = instance

        formsets = {}

        formfield_callback = self.get_formfield_callback(request)

        for prefix, config in self.inlineformset_config.items():
            config.setdefault('form', forms.ModelForm)
            config.setdefault('formfield_callback', formfield_callback)
            config.setdefault('fields', '__all__')

            cls = inlineformset_factory(self.model, extra=0, **config)
            formsets[prefix] = cls(prefix=prefix, *args, **kwargs)

        return formsets

    def get_deletion_form_instance(self, request, instance, **kwargs):
        """
        Returns a form instance which can be used to ask additional
        questions about the deletion before it actually takes place.

        Defaults to an empty form which only verifies that the deletion
        request has been POSTed.
        """
        if request.method == 'POST':
            return forms.Form(request.POST, request.FILES)
        return forms.Form()

    def save_form(self, request, form, change):
        """
        Return an unsaved instance when editing an object.
        """

        return form.save(commit=False)

    def save_model(self, request, instance, form, change):
        """
        Save an object to the database.
        """

        instance.save()

    def save_formsets(self, request, form, formsets, change):
        """
        Loop over all formsets, calling ``save_formset`` for each.
        """
        for formset in formsets.values():
            self.save_formset(request, form, formset, change)

    def save_formset(self, request, form, formset, change):
        """
        Save an individual formset
        """
        formset.save()

    def post_save(self, request, instance, form, formsets, change):
        """
        Hook for adding custom processing after forms, m2m relations
        and formsets have been saved.
        """

        pass

    # VIEW HELPERS

    def get_extra_context(self, request):
        """
        Returns a context containing the following useful variables:

        * ``verbose_name``
        * ``verbose_name_plural``
        * ``list_url``
        * ``add_url``
        * ``base_template``
        * ``adding_allowed``
        * ``search_form`` (if ``search_form_everywhere = True``)
        """
        info = app_model_label(self.model)

        return {
            'verbose_name': self.model._meta.verbose_name,
            'verbose_name_plural': self.model._meta.verbose_name_plural,
            'list_url': tryreverse('%s_%s_list' % info),
            'add_url': tryreverse('%s_%s_add' % info),
            'base_template': self.base_template,

            'adding_allowed': self.adding_allowed(request),

            'search_form': (
                self.search_form(request.GET, request=request)
                if self.search_form_everywhere else None),

            'request': request,  # Something changed with Django 1.10's
                                 # context processors...
        }

    def get_context(self, request, context):
        """
        Return a context dictionary which also contains everything from
        ``get_extra_context``.
        """
        ctx = self.get_extra_context(request)
        ctx.update(context)
        return ctx

    def render(self, request, template, context):
        """
        Render the whole shebang.
        """
        return render(request, template, context)

    def render_list(self, request, context):
        """
        Render the list view
        """
        return self.render(
            request,
            self.get_template(request, 'list'),
            self.get_context(request, context))

    def render_detail(self, request, context):
        """
        Render the detail view
        """
        return self.render(
            request,
            self.get_template(request, 'detail'),
            self.get_context(request, context))

    def render_form(self, request, context, change):
        """
        Render the add and edit views
        """
        return self.render(
            request,
            self.get_template(request, 'form'),
            self.get_context(request, context))

    def render_delete_confirmation(self, request, context):
        """
        Render the deletion confirmation page
        """
        return self.render(
            request,
            self.get_template(request, 'delete_confirmation'),
            self.get_context(request, context))

    def response_add(self, request, instance, form, formsets):
        """
        Return the response after successful addition of a new instance
        """
        self.add_message(request, 'object_created')

        if '_continue' in request.POST:
            return HttpResponseRedirect(instance.get_absolute_url() + 'edit/')

        return redirect(instance)

    def response_adding_denied(self, request):
        """
        Return the response when adding instances is denied
        """
        self.add_message(request, 'adding_denied')
        url = tryreverse('%s_%s_list' % app_model_label(self.model))
        return HttpResponseRedirect(url if url else '../../')

    def response_edit(self, request, instance, form, formsets):
        """
        Return the response after an instance has been successfully edited
        """
        self.add_message(request, 'object_updated')

        if '_continue' in request.POST:
            return HttpResponseRedirect('.')

        return redirect(instance)

    def response_editing_denied(self, request, instance):
        """
        Return the response when editing the given instance is denied
        """
        self.add_message(request, 'editing_denied')
        return redirect(instance)

    def response_delete(self, request, instance):
        """
        Return the response when an object has been successfully deleted
        """
        self.add_message(request, 'object_deleted')
        url = tryreverse('%s_%s_list' % app_model_label(self.model))
        return HttpResponseRedirect(url if url else '../../')

    def response_deletion_denied(self, request, instance):
        """
        Return the response when deleting the given instance is not allowed
        """
        self.add_message(request, 'deletion_denied')
        return redirect(instance)

    def paginate_object_list(self, request, queryset, paginate_by=10):
        """
        Helper which paginates the given object list

        Skips pagination if the magic ``all`` GET parameter is set.
        """
        paginator_obj = self.paginator_class(queryset, paginate_by)

        try:
            page = int(request.GET.get('page', '1'))
        except ValueError:
            page = 1

        try:
            page_obj = paginator_obj.page(page)
        except (paginator.EmptyPage, paginator.InvalidPage):
            page_obj = paginator_obj.page(paginator_obj.num_pages)

        if self.pagination_all_allowed and request.GET.get('all'):
            page_obj.object_list = queryset
            page_obj.show_all_objects = True
            page_obj.start_index = 1
            page_obj.end_index = paginator_obj.count

        return page_obj, paginator_obj

    # VIEWS

    def list_view(self, request, *args, **kwargs):
        """
        Handles the listing of objects

        This view knows how to paginate objects and is able
        to handle search and batch forms, too.
        """
        ctx = {}
        queryset = self.get_query_set(request, *args, **kwargs)
        ctx['root_%s' % self.template_object_list_name] = queryset

        queryset, response = self.handle_search_form(request, ctx, queryset)

        if response:
            return response

        response = self.handle_batch_form(request, ctx, queryset)
        if response:
            return response

        ctx['full_%s' % self.template_object_list_name] = queryset

        if self.paginate_by:
            page, paginator = self.paginate_object_list(
                request, queryset, self.paginate_by)

            ctx.update({
                self.template_object_list_name: page.object_list,
                'page': page,
                'paginator': paginator,
            })
        else:
            ctx[self.template_object_list_name] = queryset

        return self.render_list(request, ctx)

    def handle_search_form(self, request, ctx, queryset=None):
        """
        Must return a tuple consisting of a queryset and either a HttpResponse
        or ``None``
        """

        if queryset is None:
            queryset = self.get_query_set(request)

        if self.search_form:
            form = self.search_form(request.GET, request=request)
            if not form.is_valid():
                self.add_message(
                    request,
                    _('The search query was invalid.'),
                    level=messages.ERROR)

                if request.get_full_path().endswith('?clear=1'):
                    # No redirect loop generation
                    raise ImproperlyConfigured(
                        'Search form %r does not validate after'
                        ' clearing.' % form)

                return queryset, HttpResponseRedirect('?clear=1')

            queryset = safe_queryset_and(queryset, form.queryset(self.model))

            ctx['search_form'] = form

        return queryset, None

    def handle_batch_form(self, request, ctx, queryset):
        """
        May optionally return a HttpResponse which is directly returned to the
        browser
        """

        if not self.batch_form:
            return

        form = self.batch_form(request, queryset)
        ctx['batch_form'] = form

        if form.should_process():
            result = form.process()

            if isinstance(result, HttpResponse):
                return result

            elif hasattr(result, '__iter__'):
                messages.success(
                    request,
                    _('Processed the following items: <br>\n %s') % (
                        '<br>\n '.join(
                            force_text(item) for item in result)))

            elif result is not None:
                # Not None, but cannot make sense of it either.
                raise TypeError('Return value %r of %s.process() invalid.' % (
                    result,
                    form.__class__.__name__,
                ))

            url = tryreverse('%s_%s_list' % app_model_label(self.model))
            return HttpResponseRedirect(url if url else '.')

    def detail_view(self, request, *args, **kwargs):
        """
        Simple detail page view
        """
        instance = self.get_object_or_404(request, *args, **kwargs)

        return self.render_detail(request, {
            self.template_object_name: instance,
            'editing_allowed': self.editing_allowed(request, instance),
        })

    def adding_allowed(self, request):
        """
        By default, adding is allowed.
        """

        return True

    def process_form(self, request, instance=None, change=None):
        """
        Processes forms, used by ``add_view`` and ``edit_view``
        """

        new_instance = None
        valid = False
        ModelForm = self.get_form(request, instance, change=change)

        if request.method == 'POST':
            form = self.get_form_instance(
                request, ModelForm, instance=instance, change=change)

            if form.is_valid():
                new_instance = self.save_form(request, form, change=change)
                form_validated = True
            else:
                new_instance = self.model() if instance is None else instance
                form_validated = False

            formsets = self.get_formset_instances(
                request, instance=new_instance, change=change)

            if all_valid(formsets.values()) and form_validated:
                with transaction.atomic():
                    self.save_model(request, new_instance, form, change=change)
                    form.save_m2m()
                    self.save_formsets(request, form, formsets, change=change)
                    self.post_save(
                        request, new_instance, form, formsets, change=change)

                valid = True
        else:
            form = self.get_form_instance(
                request, ModelForm, instance=instance, change=change)
            formsets = self.get_formset_instances(
                request, instance=instance, change=change)

        return form, formsets, new_instance, valid

    def add_view(self, request):
        """
        Add view with some additional formset handling
        """
        if not self.adding_allowed(request):
            return self.response_adding_denied(request)

        form, formsets, new_instance, valid = self.process_form(
            request, change=False)

        if valid:
            return self.response_add(request, new_instance, form, formsets)

        opts = self.model._meta

        context = {
            'title': capfirst(_('Add %s') % force_text(opts.verbose_name)),
            'form': form,
            'formsets': formsets,
        }

        return self.render_form(request, context, change=False)

    def editing_allowed(self, request, instance):
        """
        By default, editing is allowed.
        """

        return True

    def edit_view(self, request, *args, **kwargs):
        """
        Edit view with some additional formset handling
        """
        instance = self.get_object_or_404(request, *args, **kwargs)

        if not self.editing_allowed(request, instance):
            return self.response_editing_denied(request, instance)

        form, formsets, new_instance, valid = self.process_form(
            request, instance=instance, change=True)

        if valid:
            return self.response_edit(request, new_instance, form, formsets)

        opts = self.model._meta

        context = {
            'title': capfirst(
                _('Change %s') % force_text(opts.verbose_name)),
            'form': form,
            'formsets': formsets,
            self.template_object_name: instance,
        }

        return self.render_form(request, context, change=True)

    def deletion_allowed(self, request, instance):
        """
        By default, deletion is not allowed.
        """

        return False

    def deletion_allowed_if_only(self, request, instance, classes):
        """
        Helper which is most useful when used inside ``deletion_allowed``

        Allows the deletion if the deletion cascade only contains
        objects from the given classes. Adds a message if deletion is
        not allowed containing details which classes are preventing
        deletion.

        Example::

            def deletion_allowed(self, request, instance):
                return self.deletion_allowed_if_only(request, instance, [
                    Ticket, TicketUpdate])
        """
        related = set(related_classes(instance))

        related.discard(self.model)
        for class_ in classes:
            related.discard(class_)

        if len(related):
            pretty_classes = [
                force_text(class_._meta.verbose_name_plural)
                for class_ in related]

            if len(pretty_classes) > 1:
                pretty_classes = ''.join((
                    ', '.join('%s' % cls for cls in pretty_classes[:-1]),
                    ugettext(' and '),
                    '%s' % pretty_classes[-1],
                ))
            else:
                pretty_classes = pretty_classes[-1]

            self.add_message(
                request,
                'deletion_denied_related',
                {'pretty_classes': pretty_classes},
                ignore=('deletion_denied',),
            )

        return not len(related)

    def save_formset_deletion_allowed_if_only(self, request, form, formset,
                                              change, classes):
        """
        Helper which has is most useful when used inside ``save_formsets``

        To use this method properly, your model has to inherit from
        ``towel.deletion.Model``. This is NOT enforced. If you ignore this, the
        instances will be deleted by ``formset.save()`` as is the default with
        Django.

        Example::

            def save_formsets(self, requset, form, formsets, change):
                # Allow deleting states when the cascade includes
                # ``SomeModel`` instances only.
                self.save_formset_deletion_allowed_if_only(
                    request, form, formsets['states'], change, [SomeModel])

                # No special handling for other formsets
                self.save_formset(request, form, formsets['otherformset'],
                    change=change)

        """
        with deletion.protect():
            self.save_formset(request, form, formset, change)

        for instance in formset.deleted_objects:
            related = set(related_classes(instance))

            related.discard(instance.__class__)
            for class_ in classes:
                related.discard(class_)

            if len(related):
                pretty_classes = [
                    force_text(class_._meta.verbose_name_plural)
                    for class_ in related]

                if len(pretty_classes) > 1:
                    pretty_classes = ''.join((
                        ', '.join(pretty_classes[:-1]),
                        ugettext(' and '),
                        pretty_classes[-1],
                    ))
                else:
                    pretty_classes = pretty_classes[-1]

                self.add_message(request, 'deletion_denied_related', {
                    'pretty_classes': pretty_classes})
            else:
                instance.delete()

    def delete_view(self, request, *args, **kwargs):
        """
        Handles deletion
        """
        obj = self.get_object_or_404(request, *args, **kwargs)

        if not self.deletion_allowed(request, obj):
            return self.response_deletion_denied(request, obj)

        form = self.get_deletion_form_instance(request, obj)

        if form.is_valid():
            obj.delete()
            return self.response_delete(request, obj)
        else:
            if not hasattr(obj, '_collected_objects'):
                related_classes(obj)

            collected_objects = [
                (key._meta, len(value))
                for key, value in obj._collected_objects.items()]

            return self.render_delete_confirmation(request, {
                'title': capfirst(_('Delete %s') % force_text(
                    self.model._meta.verbose_name)),
                self.template_object_name: obj,
                'collected_objects': collected_objects,
                'form': form,
            })


class _MVUHelper(object):
    def __init__(self, viewname_pattern, kwargs):
        self.viewname_pattern = viewname_pattern
        self.kwargs = kwargs

    def __getitem__(self, item):
        return self.url(item)

    def url(self, item, *args, **kwargs):
        kw = self.kwargs.copy()
        if args:
            kw.setdefault('args', [])
            kw['args'].extend(args)
        elif kwargs:
            kw.setdefault('kwargs', {})
            kw['kwargs'].update(kwargs)

        try:
            return reverse(self.viewname_pattern % item, **kw)
        except NoReverseMatch as e:
            try:
                return reverse(self.viewname_pattern % item)
            except NoReverseMatch:
                # Re-raise exception with kwargs; it's more informative
                raise e


class ModelViewURLs(object):
    """
    Usage::

        class MyModel(models.Model):
            urls = ModelViewURLs(lambda obj: {'pk': obj.pk})

            def get_absolute_url(self):
                return self.urls['detail']

            def get_edit_url(self):
                return self.urls['edit']

            # ... etc
    """

    def __init__(self, reverse_args_fn=None):
        """
        Despite the name, ``reverse_args_fn`` is allowed to return kwargs
        (a dict instance) too
        """
        if reverse_args_fn:
            self.reverse_args_fn = reverse_args_fn
        else:
            self.reverse_args_fn = lambda obj: {'pk': obj.pk}

    def __get__(self, obj, objtype=None):
        if not hasattr(obj, '_modelviewurls_cache'):
            kwargs = {}
            data = self.reverse_args_fn(obj)
            if isinstance(data, dict):
                kwargs['kwargs'] = data
            else:
                kwargs['args'] = data

            viewname_pattern = '%s_%s_%%s' % app_model_label(obj)

            obj._modelviewurls_cache = _MVUHelper(viewname_pattern, kwargs)
        return obj._modelviewurls_cache



"""
Assumptions
===========

* The following settings are required:

  * ``TOWEL_MT_CLIENT_MODEL``:
    The tenant model, e.g. ``clients.Client``.
  * ``TOWEL_MT_ACCESS_MODEL``:
    The model linking a Django user with a client, must have the following
    fields:

      * ``user``: Foreign key to ``auth.User``.
      * ``access``: An integer describing the access level of the given user.
        Higher numbers mean higher access. You have to define those numbers
        yourself.
      * The lowercased class name of the client model above as a foreign key
        to the client model. If your client model is named ``Customer``, the
        name of this foreign key must be ``customer``.

* All model managers have a ``for_access()`` method with a single argument,
  an instance of the access model, which returns a queryset containing only
  the objects the current user is allowed to see. The access model should be
  available as ``request.access``, which means that you are free to put
  anything there which can be understood by the ``for_access()`` methods. The
  ``request.access`` attribute is made available by the
  ``towel.mt.middleware.LazyAccessMiddleware`` middleware.
* ``towel.mt.modelview.ModelView`` automatically fills in a ``created_by``
  foreign key pointing to ``auth.User`` if it exists.
* The form classes in ``towel.mt.forms``, those being ``ModelForm``, ``Form``
  and ``SearchForm`` all require the request (the two former on initialization,
  the latter on ``post_init``). Model choice fields are postprocessed to only
  contain values from the current tenant. This does not work if you customize
  the ``choices`` field at the same time as setting the ``queryset``. If you
  do that you're on your own.
* The model authentication backend ``towel.mt.auth.ModelBackend`` also allows
  email addresses as username. It preloads the access and client model and
  assigns it to ``request.user`` if possible. This is purely a convenience --
  you are not required to use the backend.
"""

from __future__ import absolute_import, unicode_literals


def client_model():
    from django.conf import settings
    from django.db.models import loading
    return loading.get_model(*settings.TOWEL_MT_CLIENT_MODEL.split('.'))


def access_model():
    from django.conf import settings
    from django.db.models import loading
    return loading.get_model(*settings.TOWEL_MT_ACCESS_MODEL.split('.'))


class AccessDecorator(object):
    def __new__(cls):
        instance = object.__new__(cls)
        from towel import mt
        mt._access_decorator = instance
        return instance

    def __call__(self, minimal):
        """
        Apply this decorator to all views which should only be reachable by
        authenticated users with sufficient permissions::

            from towel.mt import access
            @access(access.MANAGEMENT)
            def view(request):
                # This view is only available for users with staff and
                # manager access.
        """
        from django.contrib.auth.decorators import login_required
        from django.core.exceptions import PermissionDenied
        from django.http import HttpResponse
        from django.utils.functional import wraps

        def decorator(view_func):
            @login_required
            def inner(request, *args, **kwargs):
                if not request.access:
                    return self.handle_missing(request, *args, **kwargs)

                check = self.check_access(request, minimal)

                if check is True:
                    return view_func(request, *args, **kwargs)
                elif isinstance(check, HttpResponse):
                    return check
                raise PermissionDenied('Insufficient permissions')

            fn = wraps(view_func)(inner)
            fn.original_fn = view_func
            return fn
        return decorator

    def check_access(self, request, minimal):
        return request.access.access >= minimal

    def handle_missing(self, request, *args, **kwargs):
        from django.core.exceptions import PermissionDenied
        raise PermissionDenied('Missing permissions')



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import ImproperlyConfigured

from towel.utils import safe_queryset_and


class MultitenancyMixin(object):
    def get_queryset(self):
        if self.queryset is not None:
            return safe_queryset_and(
                self.queryset,
                self.queryset.model._default_manager.for_access(
                    self.request.access))
        elif self.model is not None:
            return self.model._default_manager.for_access(
                self.request.access)
        else:
            raise ImproperlyConfigured(
                "'%s' must define 'queryset' or 'model'"
                % self.__class__.__name__)

    def get_parent_queryset(self):
        # towel.resources.inlines.ChildFormView
        return self.get_parent_class()._default_manager.for_access(
            self.request.access)

    def get_form_kwargs(self, **kwargs):
        kwargs['request'] = self.request
        return super(MultitenancyMixin, self).get_form_kwargs(**kwargs)



from __future__ import absolute_import, unicode_literals

from django import template
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _


register = template.Library()


@register.filter
def model_details(instance, fields=None):
    """
    Returns a stream of ``verbose_name``, ``value`` pairs for the specified
    model instance::

        <table>
        {% for verbose_name, value in object|model_details %}
            <tr>
                <th>{{ verbose_name }}</th>
                <td>{{ value }}</td>
            </tr>
        {% endfor %}
        </table>
    """

    if not fields:
        _fields = instance._meta.fields
    else:
        _fields = [
            instance._meta.get_field(f)
            for f in fields.split(',')]

    for f in _fields:
        if f.auto_created:
            continue

        if isinstance(f, models.ForeignKey):
            fk = getattr(instance, f.name)
            if hasattr(fk, 'get_absolute_url'):
                try:
                    value = mark_safe('<a href="%s">%s</a>' % (
                        fk.get_absolute_url(),
                        fk))
                except:  # Whatever.
                    value = fk
            else:
                value = fk

        elif f.choices:
            value = getattr(instance, 'get_%s_display' % f.name)()

        elif isinstance(f, (models.BooleanField, models.NullBooleanField)):
            value = getattr(instance, f.name)
            value = {
                True: _('yes'),
                False: _('no'),
                None: _('unknown'),
            }.get(value, value)

        else:
            value = getattr(instance, f.name)

        yield (f.verbose_name, value)



from __future__ import absolute_import, unicode_literals

from functools import reduce
import operator

from django import template
from django.utils import six
from django.utils.http import urlencode


register = template.Library()


@register.inclusion_tag('towel/_pagination.html', takes_context=True)
def pagination(context, page, paginator, where=None):
    """
    Shows pagination links::

        {% pagination current_page paginator %}

    The argument ``where`` can be used inside the pagination template to
    discern between pagination at the top and at the bottom of an object
    list (if you wish). The default object list template passes
    ``"top"`` or ``"bottom"`` to the pagination template. The default
    pagination template does nothing with this value though.
    """

    return {
        'context': context,
        'page': page,
        'paginator': paginator,
        'where': where,
    }


@register.filter
def querystring(data, exclude='page,all'):
    """
    Returns the current querystring, excluding specified GET parameters::

        {% request.GET|querystring:"page,all" %}
    """

    exclude = exclude.split(',')

    items = reduce(
        operator.add,
        (list((k, v) for v in values) for k, values
            in six.iterlists(data) if k not in exclude),
        [])

    return urlencode(sorted(items))


@register.inclusion_tag('towel/_ordering_link.html', takes_context=True)
def ordering_link(context, field, request, title='', base_url='', **kwargs):
    """
    Shows a table column header suitable for use as a link to change the
    ordering of objects in a list::

        {% ordering_link "" request title=_("Edition") %} {# default order #}
        {% ordering_link "customer" request title=_("Customer") %}
        {% ordering_link "state" request title=_("State") %}

    Required arguments are the field and the request. It is very much
    recommended to add a title too of course.

    ``ordering_link`` has an optional argument, ``base_url`` which is
    useful if you need to customize the link part before the question
    mark. The default behavior is to only add the query string, and nothing
    else to the ``href`` attribute.

    It is possible to specify a set of CSS classes too. The CSS classes
    ``'asc'`` and ``'desc'`` are added automatically by the code depending
    upon the ordering which would be selected if the ordering link were
    clicked (NOT the current ordering)::

        {% ordering_link "state" request title=_("State") classes="btn" %}

    The ``classes`` argument defaults to ``'ordering'``.
    """

    current = request.GET.get('o', '')

    # Automatically handle search form persistency
    data = request.GET.copy()
    if not data:
        form = context.get('search_form')
        if form is not None and getattr(form, 'persistency', False):
            data = form.data

    ctx = {
        'querystring': querystring(data, exclude='page,all,o'),
        'field': field,
        'used': current in (field, '-%s' % field),
        'descending': current == field,
        'title': title,
        'base_url': base_url,
    }
    ctx.update(kwargs)
    return ctx



from __future__ import absolute_import, unicode_literals

from django import template
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from towel.templatetags import towel_resources


register = template.Library()


register.inclusion_tag('towel/_pagination.html', takes_context=True)(
    towel_resources.pagination)
register.inclusion_tag('towel/_ordering_link.html', takes_context=True)(
    towel_resources.ordering_link)
register.filter(towel_resources.querystring)


@register.filter
def model_row(instance, fields):
    """
    Shows a row in a modelview object list:

    ::

        {% for object in object_list %}
            <tr>
                {% for verbose_name, field in object|model_row:"name,url" %}
                    <td>{{ field }}</td>
                {% endfor %}
            </tr>
        {% endfor %}

    """

    for name in fields.split(','):
        try:
            f = instance._meta.get_field(name)
        except models.FieldDoesNotExist:
            attr = getattr(instance, name)
            if hasattr(attr, '__call__'):
                yield (name, attr())
            else:
                yield (name, attr)
            continue

        if isinstance(f, models.ForeignKey):
            fk = getattr(instance, f.name)
            if hasattr(fk, 'get_absolute_url'):
                value = mark_safe('<a href="%s">%s</a>' % (
                    fk.get_absolute_url(),
                    fk))
            else:
                value = fk

        elif f.choices:
            value = getattr(instance, 'get_%s_display' % f.name)()

        elif isinstance(f, (models.BooleanField, models.NullBooleanField)):
            value = getattr(instance, f.name)
            value = {
                True: _('yes'),
                False: _('no'),
                None: _('unknown'),
            }.get(value, value)

        else:
            value = getattr(instance, f.name)

        yield (f.verbose_name, value)



"""
This is an experiment in splitting up the monolithic model view class into
smaller, more reusable parts, and using class-based views at the same time
(not the generic class based views, though)

The basic idea is that creating a new instance for every request and having
different classes handling different types of requests (listing, details,
editing etc.) are both really good ideas. That's what Django's class-based
views do, and they do it quite well. However, sharing functionality between
those different classes is hard: Adding limits to querysets centrally, adding
a common set of list/detail/crud views, adding the same context variables to
all views for the same Django model are all harder than they should be.

That's where ``towel.resources`` really shines. All views inherit from
``ModelResourceView`` which in turn inherits all functionality of
``django.views.generic.base.TemplateView``. Many methods such as
``get_queryset``, ``get_context_data``, ``get_form_kwargs`` etc. are closely
modelled after the generic class-based views of Django itself, the inheritance
hierarchy is a lot simpler and we use less mixins, and less inversion of
control which should make code written with ``towel.resources`` easier to
understand and follow.

At least that's one of our goals here.
"""

from __future__ import absolute_import, unicode_literals

import json

from django import forms
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.core.urlresolvers import NoReverseMatch
from django.forms.models import modelform_factory, model_to_dict
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect
from django.utils.encoding import force_text
from django.utils.text import capfirst
from django.utils.translation import ugettext as _
from django.views.generic.base import TemplateView

from towel.forms import BatchForm, towel_formfield_callback
from towel.paginator import Paginator, EmptyPage, InvalidPage
from towel.utils import (
    app_model_label, changed_regions, related_classes, safe_queryset_and)


class ModelResourceView(TemplateView):
    """
    This is the base class of all views in ``towel.resources``.
    """

    #: The ``base_template`` variable is passed into the template context
    #: when rendering any templates. It's most useful when adding a base
    #: template which should always be used when rendering templates related
    #: to a single model. A practical example might be adding the same sidebar
    #: to all resources related to a specific model.
    base_template = 'base.html'

    #: The model. Required.
    model = None

    #: Overrides the queryset used for rendering. Override ``get_queryset``
    #: below if you have more advanced needs.
    queryset = None

    #: Part of the template name. Is set in most view classes to a sane value
    #: such as ``_list``, ``_detail``, ``_form`` or something similar.
    template_name_suffix = None

    def url(self, item, *args, **kwargs):
        """
        Helper for reversing URLs related to the resource model. Works the
        same way as ``towel.modelview.ModelViewURLs`` (and is most useful
        if used together).

        Usage examples::

            self.url('list')

            self.url('edit', pk=self.object.pk)
            # equals self.object.urls.url('edit') if using ModelViewURLs
        """
        fail_silently = kwargs.pop('fail_silently', False)

        try:
            if getattr(self, 'object', None):
                return self.object.urls.url(item, *args, **kwargs)
            return self.model().urls.url(item, *args, **kwargs)
        except NoReverseMatch:
            if not fail_silently:
                raise
            return None

    def get_title(self):
        """
        Returns a sane value for the ``title`` template context variable.
        """
        return None

    def get_context_data(self, **kwargs):
        """
        Fills the standard context with default variables useful for all
        model resource views:

        - ``base_template``: Described above.
        - ``verbose_name`` and ``verbose_name_plural``: Current model.
        - ``view``: The view instance.
        - ``add_url`` and ``list_url``: The mose important URLs for the model.
        - ``title``: Described above.
        """
        opts = self.model._meta
        context = {
            'base_template': self.base_template,
            'verbose_name': opts.verbose_name,
            'verbose_name_plural': opts.verbose_name_plural,
            'view': self,

            'add_url': self.url('add', fail_silently=True),
            'list_url': self.url('list', fail_silently=True),
        }
        title = self.get_title()
        if title:
            context['title'] = title
        context.update(kwargs)
        return context

    def get_template_names(self):
        """
        Returns a list of template names related to the current model and view:

        - ``self.template_name`` if it's set.
        - ``<app_label>/<model_name><template_name_suffix>.html
        - ``resources/object<template_name_suffix>.html
        """
        names = [
            '{0}/{1}{2}.html'.format(
                *(app_model_label(self.model) + (self.template_name_suffix,))),
            'resources/object{0}.html'.format(self.template_name_suffix),
        ]
        if self.template_name:
            names.insert(0, self.template_name)
        return names

    def get_queryset(self):
        """
        Returns the queryset used everywhere.

        Defaults to ``self.model._default_manager.all()``.
        """
        if self.queryset is not None:
            return self.queryset._clone()
        elif self.model is not None:
            return self.model._default_manager.all()
        else:
            raise ImproperlyConfigured("'%s' must define 'queryset' or 'model'"
                                       % self.__class__.__name__)

    def get_object(self):
        """
        Returns a single object for detail views or raises ``Http404``.

        The default implementation passes all keyword arguments extracted from
        the URL into ``get_object_or_404``.
        """
        return get_object_or_404(self.get_queryset(), **self.kwargs)

    def allow_add(self, silent=True):
        """
        Whether adding objects should be allowed. Defaults to ``True``.

        If ``silent=False`` you can optionally add a message in your own
        implementation.
        """
        return True

    def allow_edit(self, object=None, silent=True):
        """
        Whether editing objects should be allowed. Defaults to ``True``.

        Should determine whether editing objects is allowed under any
        circumstances if ``object=None``.
        """
        return True

    def allow_delete(self, object=None, silent=True):
        """
        Whether deleting objects should be allowed. Defaults to ``False``.

        Should determine whether editing objects is allowed under any
        circumstances if ``object=None``.

        Adds a message that deletion is not allowed when ``silent=False``.
        """
        if not silent:
            opts = self.model._meta
            if object is None:
                messages.error(self.request, _(
                    'You are not allowed to'
                    ' delete %(verbose_name_plural)s.'
                ) % opts.__dict__)
            else:
                messages.error(self.request, _(
                    'You are not allowed to'
                    ' delete this %(verbose_name)s.'
                ) % opts.__dict__)
        return False

    def allow_delete_if_only(self, object, related=(), silent=True):
        """
        This helper is most useful when used inside ``allow_delete``. It can
        be used to easily determine whether there are related instances which
        would be deleted as well.

        Returns ``True`` if the classes only belong to the model itself and
        to the classes mentioned in ``related``.
        """
        classes = set(related_classes(object)).difference(
            (self.model,), related)
        if not classes:
            return True
        if not silent:
            messages.error(
                self.request,
                _('Deletion not allowed because of related objects: %s') % (
                    ', '.join(
                        force_text(cls._meta.verbose_name_plural)
                        for cls in classes
                    ),
                )
            )
        return False


class ListView(ModelResourceView):
    """
    View used for listing objects. Has support for pagination, search forms
    and batch actions similar to the actions built into Django's admin
    interface.
    """

    #: Objects per page. Defaults to ``None`` which means no pagination.
    paginate_by = None

    #: Search form class.
    search_form = None

    #: ``object_list.html`` it is.
    template_name_suffix = '_list'

    def get_paginate_by(self, queryset):
        """
        Returns the value of ``self.paginate_by``.
        """
        # if self.paginate_all_allowed and self.request.GET.get('all'):
        #     return None
        return self.paginate_by

    def get_context_data(self, object_list=None, **kwargs):
        """
        Adds ``object_list`` to the context, and ``page`` and ``paginator``
        as well if paginating.
        """
        context = super(ListView, self).get_context_data(
            object_list=object_list, **kwargs)

        if object_list is not None:
            paginate_by = self.get_paginate_by(object_list)
            if paginate_by:
                paginator = Paginator(object_list, paginate_by)

                try:
                    page = int(self.request.GET.get('page'))
                except (TypeError, ValueError):
                    page = 1
                try:
                    page = paginator.page(page)
                except (EmptyPage, InvalidPage):
                    page = paginator.page(paginator.num_pages)

                context.update({
                    'object_list': page.object_list,
                    'page': page,
                    'paginator': paginator,
                })

        return context

    def get(self, request, *args, **kwargs):
        """
        Handles the search form and batch action handling.
        """
        self.object_list = self.get_queryset()
        context = {}

        if self.search_form:
            form = self.search_form(self.request.GET, request=self.request)
            if not form.is_valid():
                messages.error(
                    self.request, _('The search query was invalid.'))
                return HttpResponseRedirect('?clear=1')
            self.object_list = safe_queryset_and(
                self.object_list,
                form.queryset(self.model),
            )
            context['search_form'] = form

        context.update(self.get_context_data(object_list=self.object_list))

        actions = self.get_batch_actions()
        if actions:
            form = BatchForm(self.request, self.object_list)
            form.actions = actions
            form.fields['action'] = forms.ChoiceField(
                label=_('Action'),
                choices=[('', '---------')] + [row[:2] for row in actions],
                widget=forms.HiddenInput,
            )
            context['batch_form'] = form

            if form.should_process():
                action = form.cleaned_data.get('action')
                name, title, fn = [a for a in actions if action == a[0]][0]
                result = fn(form.batch_queryset)
                if isinstance(result, HttpResponse):
                    return result
                elif hasattr(result, '__iter__'):
                    messages.success(
                        self.request,
                        _(
                            '<p>Processed the following items:</p>'
                            ' <ul><li>%s</li></ul>'
                        ) % (
                            '</li><li>'.join(
                                force_text(item) for item in result
                            )
                        )
                    )
                elif result is not None:
                    # Not None, but cannot make sense of it either.
                    raise TypeError('Return value %r of %s invalid.' % (
                        result, fn.__name__))

                return redirect(self.url('list'))

        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        """
        Calls ``self.get`` because of the batch action handling.
        """
        return self.get(request, *args, **kwargs)

    def get_batch_actions(self):
        """
        Returns a list of batch action tuples ``(key, name, handler_fn)``

        * ``key``: Something nice, such as ``delete_selected``.
        * ``name``: Will be shown in the dropdown.
        * ``handler_fn``: Callable. Receives the request and the queryset.
        """
        return [
            ('delete_selected', _('Delete selected'), self.delete_selected),
        ]

    def batch_action_hidden_fields(self, queryset, additional=[]):
        """
        Returns a blob of HTML suitable for jumping back into the batch
        action handler. Most useful for batch action handlers needing to
        present a confirmation and/or form page to the user.

        See ``delete_selected`` below for the usage.
        """
        post_values = [('batchform', 1)] + additional + [
            ('batch_%s' % item.pk, '1') for item in queryset]

        return '\n'.join(
            '<input type="hidden" name="%s" value="%s">' % item
            for item in post_values)

    def delete_selected(self, queryset):
        """
        Action which deletes all selected items provided:

        - Their deletion is allowed.
        - Confirmation is given on a confirmation page.
        """
        allowed = [self.allow_delete(item) for item in queryset]
        queryset = [item for item, perm in zip(queryset, allowed) if perm]

        if not queryset:
            messages.error(self.request, _(
                'You are not allowed to delete any'
                ' object in the selection.'))
            return

        elif not all(allowed):
            messages.warning(self.request, _(
                'Deletion of some objects not allowed. Those have been'
                ' excluded from the selection already.'))

        if 'confirm' in self.request.POST:
            messages.success(self.request, _('Deletion successful.'))
            # Call all delete() methods individually
            [item.delete() for item in queryset]
            return

        context = super(ListView, self).get_context_data(
            title=_('Delete selected'),
            action_queryset=queryset,
            action_hidden_fields=self.batch_action_hidden_fields(queryset, [
                ('batch-action', 'delete_selected'),
                ('confirm', 1),
            ]),
        )
        self.template_name_suffix = '_action'
        return self.render_to_response(context)


class DetailView(ModelResourceView):
    """
    Detail view. Nuff said.
    """
    template_name_suffix = '_detail'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    @classmethod
    def render_regions(cls, view, **kwargs):
        """
        This is mostly helpful when using ``{% region %}`` template tags. It
        returns all regions when rendering the detail page of the passed
        view. Those regions still have to be filtered using ``changed_regions``
        if you do not want to exchange all snippets.

        When editing data which is used to render a detail page of the same
        or of a different, related model instance, rendering the detail page
        and extracting (potentially) changed parts is necessary for
        live-updating the affected HTML snippets in the frontend.

        This class method achieves exactly that.

        Usage examples::

            # When editing the same model as is on display
            regions = DetailView.render_regions(self)

            # When using towel.resources.inlines
            regions = DetailView.render_regions(self,
                model=self.get_parent_class(),
                object=self.parent,
                )
        """
        self = cls()
        self.request = view.request
        self.model = view.model
        self.object = view.object  # This is, of course, not always correct.
        for key, value in kwargs.items():
            setattr(self, key, value)

        regions = {}
        context = self.get_context_data(object=self.object, regions=regions)
        self.render_to_response(context).render()
        return regions


class FormView(ModelResourceView):
    """
    Base class for all views handling forms (creations and updates).
    """

    #: Base form class used for editing objects. The default implementation
    #: of ``get_form_class`` below always uses ``modelform_factory`` with
    #: a custom ``formfield_callback``.
    form_class = forms.ModelForm

    #: The object being edited (or ``None`` if creating a new object).
    object = None

    #: ``object_form.html`` should be enough for everyone.
    template_name_suffix = '_form'

    def get_title(self):
        """
        Returns titles suitable for adding or editing objects depending on
        whether ``self.object`` is set or not.
        """
        if self.object and self.object.pk:
            return capfirst(_('Edit %s') % self.object)
        return capfirst(_('Add %s') % self.model._meta.verbose_name)

    def get_form_kwargs(self, **kwargs):
        """
        Returns a dictionary suitable for initializing a model form instance
        when using the dictionary as keyword arguments.

        All passed keyword arguments are added to the dictionary, and may
        be used to override any of ``data``, ``files`` and ``instance``.
        """
        kw = {'instance': self.object}
        if self.request.method in ('POST', 'PUT'):
            kw.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        kw.update(kwargs)
        return kw

    def get_form_class(self):
        """
        Returns the form class used in the view.
        """
        return modelform_factory(
            self.model,
            form=self.form_class,
            fields='__all__',
            formfield_callback=towel_formfield_callback)

    def get_form(self):
        """
        Returns the form instance used in the view.
        """
        return self.get_form_class()(**self.get_form_kwargs())

    def form_valid(self, form):
        """
        Processes the form if validation succeeded.

        The default implementation saves the form first and redirects to
        the detail URL of the returned instance.
        """
        self.object = form.save()
        messages.success(
            self.request,
            _('The %(verbose_name)s has been successfully saved.') %
            self.object._meta.__dict__,
        )
        return redirect(self.object)

    def form_invalid(self, form):
        """
        Processes the form if validation failed.

        The default implementation renders the form again.
        """
        context = self.get_context_data(form=form, object=self.object)
        return self.render_to_response(context)


class AddView(FormView):
    """
    View class handling the creation of new items.

    Checks whether adding objects is allowed first and redirects to the
    list URL for the current model if not.
    """
    def get(self, request, *args, **kwargs):
        if not self.allow_add(silent=False):
            return redirect(self.url('list'))
        form = self.get_form()
        return self.render_to_response(self.get_context_data(form=form))

    def post(self, request, *args, **kwargs):
        if not self.allow_add(silent=False):
            return redirect(self.url('list'))
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        return self.form_invalid(form)


class EditView(FormView):
    """
    View class handling the editing of existing items.

    Fetches the instance from the database first and checks whether editing
    it is allowed next. The permission check is done second because the result
    might depend on the exact object.

    The return value of not found versus permission denied exposes some
    additional information right now. This will not necessarily stay like that
    in the future, do not rely on it.
    """
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.allow_edit(self.object, silent=False):
            return redirect(self.object)
        form = self.get_form()
        context = self.get_context_data(form=form, object=self.object)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.allow_edit(self.object, silent=False):
            return redirect(self.object)
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        return self.form_invalid(form)


class LiveUpdateAfterEditMixin(object):
    """
    Only uses the editlive mechanism for updating. The edit step happens
    inside a standard modal.
    """
    def form_valid(self, form):
        self.object = form.save()

        regions = DetailView.render_regions(self)
        data = {'!form-errors': {}}
        data.update(changed_regions(regions, form.changed_data))
        return HttpResponse(json.dumps(data), content_type='application/json')


class LiveFormView(LiveUpdateAfterEditMixin, FormView):
    """
    View most useful if the detail page contains editlive input fields. This
    is the POST-only resource for updates.

    Only supports updating fields on the model itself, and no many to many
    fields either.
    """
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.allow_edit(self.object, silent=True):
            raise PermissionDenied

        form_class = self.get_form_class()
        data = model_to_dict(
            self.object,
            fields=form_class._meta.fields,
            exclude=form_class._meta.exclude)

        for key, value in request.POST.items():
            data[key] = value

        form = form_class(**self.get_form_kwargs(data=data))

        if form.is_valid():
            return self.form_valid(form)

        return HttpResponse(
            json.dumps({'!form-errors': dict(form.errors)}),
            content_type='application/json')


class PickerView(ModelResourceView):
    """
    View handling a picker opened in a modal layer. Requires the editlive
    Javascript code to be loaded for searches to work.
    """
    template_name_suffix = '_picker'

    def get_title(self):
        return capfirst(_('Select a %s') % self.model._meta.verbose_name)

    def get(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        regions = None
        query = request.GET.get('query')

        if query is not None:
            self.object_list = safe_queryset_and(
                self.object_list,
                self.model.objects._search(query))
            regions = {}

        context = self.get_context_data(
            object_list=self.object_list,
            regions=regions)
        response = self.render_to_response(context)

        if query is not None:
            response.render()
            data = changed_regions(regions, ['object_list'])
            data['!keep'] = True  # Keep modal open
            return HttpResponse(
                json.dumps(data),
                content_type='application/json')

        return response


class DeleteView(ModelResourceView):
    """
    View class handling the deletion of items.

    Fetches the instance from the database first and checks whether deletion
    it is allowed next. The permission check is done second because the result
    might depend on the exact object.

    The return value of not found versus permission denied exposes some
    additional information right now. This will not necessarily stay like that
    in the future, do not rely on it.

    This view uses a form for the confirmation. The default form is empty and
    validates for all POST requests. You can optionally specify your own form
    if additional confirmation is required.
    """
    #: ``object_delete_confirmation.html``.
    template_name_suffix = '_delete_confirmation'

    #: The form class used for deletions.
    deletion_form_class = forms.Form

    def get_title(self):
        return capfirst(_('Delete %s') % self.object)

    def get_deletion_form(self):
        """
        Returns the form instance. Customize this method if you need custom
        form initialization; ``get_form_class`` and ``get_form_kwargs`` are
        not available for deletions for simplicity.
        """
        if self.request.method == 'POST':
            return self.deletion_form_class(self.request.POST)
        return self.deletion_form_class()

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.allow_delete(self.object, silent=False):
            return redirect(self.object)
        form = self.get_deletion_form()
        context = self.get_context_data(object=self.object, form=form)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.allow_delete(self.object, silent=False):
            return redirect(self.object)
        form = self.get_deletion_form()
        if form.is_valid():
            return self.deletion_form_valid(form)
        return self.deletion_form_invalid(form)

    def deletion_form_valid(self, form):
        """
        On successful form validation, the object is deleted and the user is
        redirected to the list view of the model.
        """
        self.object.delete()
        messages.success(
            self.request,
            _('The %(verbose_name)s has been successfully deleted.') %
            self.object._meta.__dict__,
        )
        return redirect(self.url('list'))

    def deletion_form_invalid(self, form):
        context = self.get_context_data(object=self.object, form=form)
        return self.render_to_response(context)



from __future__ import absolute_import, unicode_literals

import json

from django import forms
from django.db import models
from django.db.models import ObjectDoesNotExist
try:
    from django.forms.utils import flatatt
except ImportError:
    from django.forms.util import flatatt
from django.http import HttpResponse, QueryDict
from django.utils import six
from django.utils.encoding import force_text, force_bytes
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from towel import quick


class BatchForm(forms.Form):
    """
    This form class can be used to provide batch editing functionality
    in list views, similar to Django's admin actions.

    You have to implement your batch processing in the ``_context()``
    method. This method only receives one parameter, a queryset which
    is already filtered according to the selected items on the list view.
    Additionally, the current request is available as an attribute of the
    form instance, ``self.request``.

    The method ``process(self)`` may have the following return values:

    - A ``dict`` instance: Will be merged into the template context.
    - A ``HttpResponse`` instance: Will be returned directly to the client.
    - An iterable: The handler assumes successful processing of all objects
      contained in the iterable.
    - Nothing: Nothing happens.

    Usage example::

        class AddressBatchForm(BatchForm):
            subject = forms.CharField()
            body = forms.TextField()

            def process(self):
                # Form validation has already been taken care of
                subject = self.cleaned_data.get('subject')
                body = self.cleaned_data.get('body')

                if not (subject and body):
                    return {}

                sent = 0
                for item in self.batch_queryset:
                    send_mail(subject, body, settings.DEFAULT_SENDER,
                        [item.email])
                    sent += 1
                if sent:
                    messages.success(self.request, 'Sent %s emails.' % sent)

                return self.batch_queryset

        def addresses(request):
            queryset = Address.objects.all()
            batch_form = AddressBatchForm(request, queryset)
            ctx = {'addresses': queryset}

            if batch_form.should_process():
                result = form.process()
                if isinstance(result, HttpResponse):
                    return result
                elif isinstance(result, dict):
                    ctx.update(result)
                elif hasattr(result, '__iter__'):
                    messages.success(request,
                        _('Processed the following items: %s') % (
                            ', '.join(force_text(item) for item in result)))

                return HttpResponseRedirect('.')

            return render(request, 'addresses.html', ctx)

    Template code::

        {% load towel_batch_tags %}
        <form method="post" action=".">
            <ul>
            {% for address in addresses %}
                <li>
                {% batch_checkbox address.id batch_form %}
                {{ address }}
                </li>
            {% endfor %}
            </ul>

            {# Required! Otherwise, ``BatchForm.process`` does nothing. #}
            <input type="hidden" name="batchform" value="1" />

            <table>
                {{ batch_form }}
            </table>
            <button type="submit">Send mail to selected</button>
        </form>
    """

    _process = False
    ids = []

    def __init__(self, request, queryset, *args, **kwargs):
        kwargs.setdefault('prefix', 'batch')

        self.request = request
        self.queryset = queryset

        if request.method == 'POST' and 'batchform' in request.POST:
            self._process = True
            super(BatchForm, self).__init__(
                request.POST, request.FILES, *args, **kwargs)
        else:
            super(BatchForm, self).__init__(*args, **kwargs)

    def clean(self):
        """
        Cleans the batch form fields and checks whether at least one item
        had been selected.
        """
        data = super(BatchForm, self).clean()

        post_data = self.request.POST
        self.ids = [
            pk for pk in self.queryset.values_list('id', flat=True)
            if post_data.get('batch_%s' % pk)]

        if not self.ids:
            raise forms.ValidationError(_('No items selected'))

        return data

    def should_process(self):
        """
        Returns true when the submitted form was the batch form, and the
        batch form is valid.
        """
        return self._process and self.is_valid()

    @cached_property
    def batch_queryset(self):
        """
        Returns the queryset containing only items that have been selected
        for batch processing.
        """
        return self.queryset.filter(id__in=self.ids)

    def process(self):  # pragma: no cover
        """
        Actually processes the batch form submission. Override this with
        your own behavior.

        Batch forms may return the following types here (they are handled
        by ``ModelView.handle_batch_form``:

        - A ``HttpResponse``:
          Will be returned directly to the user.
        - An iterable:
          A success message will be generated containing all items in the
          iterable.
        """
        if hasattr(self, '_context'):
            import warnings
            warnings.warn(
                'The batch form \'%s.%s\' is still using \'_context\'.'
                ' Switch to using the new \'process\' method now!' % (
                    self.__class__.__module__,
                    self.__class__.__name__,
                ),
                DeprecationWarning,
            )

            ctx = self._context(self.batch_queryset)
            if 'response' in ctx:
                return ctx['response']
            return ctx

        raise NotImplementedError(
            'BatchForm.process has no default implementation.')


class SearchForm(forms.Form):
    """
    Supports persistence of searches (stores search in the session). Requires
    not only the GET parameters but the request object itself to work
    correctly.

    Usage example::

        class AddressManager(SearchManager):
            search_fields = ('first_name', 'last_name', 'address', 'email',
                'city', 'zip_code', 'created_by__email')

        class Address(models.Model):
            ...

            objects = AddressManager()

        class AddressSearchForm(SearchForm):
            orderings = {
                '': ('last_name', 'first_name'), # Default
                'dob': 'dob', # Sort by date of birth
                'random': lambda queryset: queryset.order_by('?'),
                }
            is_person = forms.NullBooleanField()

        def addresses(request):
            search_form = AddressSearchForm(request.GET, request=request)
            queryset = search_form.queryset(Address)
            ctx = {
                'addresses': queryset,
                'search_form': search_form,
                }
            return render(request, 'addresses.html', ctx)

    .. warning::

        All fields in the form need to have ``required=False`` set. Otherwise,
        form validation would already fail on the first visit on the list
        page (which would kind of defeat the purpose of a search form).

    Template code::

        <form method="get" action=".">
            <input type="hidden" name="s" value="1"> <!-- SearchForm search -->
            <table>
                {{ search_form }}
            </table>
            <button type="submit">Search</button>
        </form>

        {% for address in addresses %}
            ...
        {% endfor %}
    """

    #: Fields which are always excluded from automatic filtering
    #: in ``apply_filters``
    always_exclude = ('s', 'query', 'o')

    #: Default field values - used if not overridden by the user
    default = {}

    #: Ordering specification
    orderings = {}

    #: Quick rules, a list of (regex, mapper) tuples
    quick_rules = []

    #: Search form active?
    s = forms.CharField(required=False, widget=forms.HiddenInput(),
                        initial='1')

    #: Current ordering
    o = forms.CharField(required=False, widget=forms.HiddenInput())

    #: Full text search query
    query = forms.CharField(
        label=_('Query'),
        required=False,
        widget=forms.TextInput(attrs={'placeholder': _('Query')}))

    def __init__(self, data, *args, **kwargs):
        # Are the results filtered in any way?
        self.filtered = True
        # Is a persisted search active?
        self.persistency = False

        request = kwargs.pop('request')
        self.original_data = data
        super(SearchForm, self).__init__(
            self.prepare_data(data, request),
            *args, **kwargs)
        self.persist(request)
        self.post_init(request)

    def prepare_data(self, data, request):
        """
        Fill in default values from ``default`` if they aren't provided by
        the user.
        """

        if not self.default:
            return data

        data = data.copy()
        for key, value in self.default.items():
            if key not in data:
                if hasattr(value, '__call__'):
                    value = value(request)

                if hasattr(value, '__iter__'):
                    data.setlist(key, value)
                else:
                    data[key] = value
        return data

    def post_init(self, request):
        """
        Hook for customizations.
        """

        pass

    def persist(self, request):
        """
        Persist the search in the session, or load saved search if user
        isn't searching right now.
        """

        session_key = 'sf_%s.%s' % (
            self.__class__.__module__,
            self.__class__.__name__)

        if 'clear' in request.GET or 'n' in request.GET:
            if session_key in request.session:
                del request.session[session_key]

        if self.original_data and (
                set(self.original_data.keys()) & set(self.fields.keys())):
            data = self.data.copy()
            if 's' in data:
                del data['s']
                request.session[session_key] = data.urlencode()

        elif request.method == 'GET' and 's' not in request.GET:
            # try to get saved search from session
            if session_key in request.session:
                session_data = force_bytes(request.session[session_key])
                self.data = QueryDict(session_data, encoding='utf-8')
                self.persistency = True

            else:
                self.filtered = False

        elif request.method == 'POST' and 's' not in request.POST:
            # It wasn't the search form which was POSTed, hopefully :-)
            self.filtered = False

    def searching(self):
        """
        Returns ``searching`` for use as CSS class if results are filtered
        by this search form in any way.
        """

        if self.persistency or self.safe_cleaned_data.get('s'):
            return 'searching'
        return ''

    @property
    def safe_cleaned_data(self):
        """
        Safely return a dictionary of values, even if search form isn't valid.
        """

        self.is_valid()
        try:
            return self.cleaned_data.copy()
        except AttributeError:
            return {}

    def fields_iterator(self):
        """
        Yield all additional search fields.
        """

        skip = ('query', 's', 'o')

        for field in self:
            if field.name not in skip:
                yield field

    def apply_filters(self, queryset, data, exclude=()):
        """
        Automatically apply filters

        Uses form field names for ``filter()`` argument construction.
        """

        exclude = list(exclude) + list(self.always_exclude)

        for field in self.fields.keys():
            if field in exclude:
                continue

            value = data.get(field)
            if (value and hasattr(value, '__iter__')
                    and not isinstance(value, six.string_types)):
                queryset = queryset.filter(**{'%s__in' % field: value})
            elif value or value is False:
                queryset = queryset.filter(**{field: value})

        if self.quick_rules:
            quick_only = set(data.keys()) - set(self.fields.keys())
            for field in quick_only:
                if field in exclude:
                    continue

                if field.endswith('_') and (
                        field[:-1] in quick_only or field[:-1] in self.fields):
                    # Either ``quick.model_mapper`` wanted to trick us and
                    # added the model instance, too, or the quick mechanism
                    # filled an existing form field which means that the
                    # attribute has already been handled above.
                    continue

                value = data.get(field)
                if value is not None:
                    queryset = queryset.filter(**{field: value})

        return queryset

    def apply_ordering(self, queryset, ordering=None):
        """
        Applies ordering if the value in ``o`` matches a key in
        ``self.orderings``. The ordering may also be reversed,
        in which case the ``o`` value should be prefixed with
        a minus sign.
        """
        if ordering is None:
            return queryset

        if ordering and ordering[0] == '-':
            order_by, desc = ordering[1:], True
        else:
            order_by, desc = ordering, False

        if order_by not in self.orderings:
            # Ignore ordering request if unknown
            return queryset

        order_by = self.orderings[order_by]

        if hasattr(order_by, '__call__'):
            queryset = order_by(queryset)
        elif isinstance(order_by, (list, tuple)):
            queryset = queryset.order_by(*order_by)
        else:
            queryset = queryset.order_by(order_by)

        if desc:
            return queryset.reverse()
        return queryset

    def query_data(self):
        """
        Return a fulltext query and structured data which can be converted into
        simple filter() calls
        """

        if not hasattr(self, '_query_data_cache'):
            data = self.safe_cleaned_data

            if self.quick_rules:
                data, query = quick.parse_quickadd(
                    data.get('query'),
                    self.quick_rules)
                query = ' '.join(query)

                # Data in form fields overrides any quick specifications
                for k, v in self.safe_cleaned_data.items():
                    if v is not None:
                        data[k] = v
            else:
                query = data.get('query')

            self._query_data_cache = query, data
        return self._query_data_cache

    def queryset(self, model):
        """
        Return the result of the search
        """

        query, data = self.query_data()
        queryset = model.objects.search(query)
        queryset = self.apply_filters(queryset, data)
        return self.apply_ordering(queryset, data.get('o'))


class WarningsForm(forms.BaseForm):
    """
    Form subclass which allows implementing validation warnings

    In contrast to Django's ``ValidationError``, these warnings may
    be ignored by checking a checkbox.

    The warnings support consists of the following methods and properties:

    * ``WarningsForm.add_warning(<warning>)``: Adds a new warning message
    * ``WarningsForm.warnings``: A list of warnings or an empty list if there
      are none.
    * ``WarningsForm.is_valid()``: Overridden ``Form.is_valid()``
      implementation which returns ``False`` for otherwise valid forms with
      warnings, if those warnings have not been explicitly ignored (by checking
      a checkbox or by passing ``ignore_warnings=True`` to ``is_valid()``.
    * An additional form field named ``ignore_warnings`` is available - this
      field should only be displayed if ``WarningsForm.warnings`` is non-emtpy.
    """
    def __init__(self, *args, **kwargs):
        super(WarningsForm, self).__init__(*args, **kwargs)

        self.fields['ignore_warnings'] = forms.BooleanField(
            label=_('Ignore warnings'), required=False)
        self.warnings = []

    def add_warning(self, warning):
        """
        Adds a new warning, should be called while cleaning the data
        """
        self.warnings.append(warning)

    def is_valid(self, ignore_warnings=False):
        """
        ``is_valid()`` override which returns ``False`` for forms with warnings
        if these warnings haven't been explicitly ignored
        """
        if not super(WarningsForm, self).is_valid():
            return False

        if self.warnings and not (
                ignore_warnings or self.cleaned_data.get('ignore_warnings')):
            return False

        return True


class StrippedInputMixin(object):
    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        if isinstance(value, six.string_types):
            return value.strip()
        return value


class StrippedTextInput(StrippedInputMixin, forms.TextInput):
    """
    ``TextInput`` form widget subclass returning stripped contents only
    """
    pass


class StrippedTextarea(StrippedInputMixin, forms.Textarea):
    """
    ``Textarea`` form widget subclass returning stripped contents only
    """
    pass


def towel_formfield_callback(field, **kwargs):
    """
    Use this callback as ``formfield_callback`` if you want to use stripped
    text inputs and textareas automatically without manually specifying the
    widgets. Adds a ``dateinput`` class to date and datetime fields too.
    """

    if isinstance(field, models.CharField) and not field.choices:
        kwargs['widget'] = StrippedTextInput()
    elif isinstance(field, models.TextField):
        kwargs['widget'] = StrippedTextarea()
    elif isinstance(field, models.DateTimeField):
        kwargs['widget'] = forms.DateTimeInput(attrs={'class': 'dateinput'})
    elif isinstance(field, models.DateField):
        kwargs['widget'] = forms.DateInput(attrs={'class': 'dateinput'})

    return field.formfield(**kwargs)


#: Backwards compatibility, provide the function under the old name
#: ``stripped_formfield_callback`` too.
def stripped_formfield_callback(field, **kwargs):
    import warnings
    warnings.warn(
        'stripped_formfield_callback has been renamed to'
        ' towel_formfield_callback, please start using the new name.',
        DeprecationWarning, stacklevel=2)
    return towel_formfield_callback(field, **kwargs)


def autocompletion_response(queryset, limit=10):
    """
    Helper which returns a ``HttpResponse`` list of instances in a format
    suitable for consumption by jQuery UI Autocomplete, respectively
    ``towel.forms.ModelAutocompleteWidget``.
    """
    return HttpResponse(
        json.dumps([
            {
                'label': force_text(instance),
                'value': instance.pk,
            } for instance in queryset[:limit]
        ]),
        content_type='application/json',
    )


class ModelAutocompleteWidget(forms.TextInput):
    """
    Model autocompletion widget using jQuery UI Autocomplete

    Supports both querysets and JSON-returning AJAX handlers as data
    sources. Use as follows::

        class MyForm(forms.ModelForm):
            customer = forms.ModelChoiceField(Customer.objects.all(),
                widget=ModelAutocompleteWidget(url='/customers/search_ajax/'),
                )
            type = forms.ModelChoiceField(Type.objects.all(),
                widget=ModelAutocompleteWidget(queryset=Type.objects.all()),
                )

    You need to make sure that the jQuery UI files are loaded correctly
    yourself.
    """

    def __init__(self, attrs=None, url=None, queryset=None):
        assert (url is None) != (queryset is None), (
            'Provide either url or queryset')

        self.url = url
        self.queryset = queryset
        super(ModelAutocompleteWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None, choices=()):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, type='hidden', name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_text(self._format_value(value))

        hidden = '<input%s />' % flatatt(final_attrs)

        final_attrs['type'] = 'text'
        final_attrs['id'] += '_ac'
        del final_attrs['name']

        try:
            instance = self.choices.queryset.get(pk=value)
            final_attrs['value'] = force_text(instance)
        except (ObjectDoesNotExist, ValueError, TypeError):
            final_attrs['value'] = ''

        if self.is_required:
            ac = '<input%s />' % flatatt(final_attrs)
        else:
            final_attrs.setdefault('class', '')
            final_attrs['class'] += ' ac_nullable'

            ac = (
                ' <a href="#" id="%(id)s_cl" class="ac_clear">'
                ' %(text)s</a>' % {
                    'id': final_attrs['id'][:-3],
                    'text': _('clear'),
                }
            ) + ('<input%s />' % flatatt(final_attrs))

        js = '''<script type="text/javascript">
$(function() {
    $('#%(id)s_ac').autocomplete({
        source: %(source)s,
        focus: function(event, ui) {
            $('#%(id)s_ac').val(ui.item.label);
            return false;
        },
        select: function(event, ui) {
            $('#%(id)s').val(ui.item.value).trigger('change');
            $('#%(id)s_ac').val(ui.item.label);
            return false;
        }
    }).bind('focus', function() {
        this.select();
    }).bind('blur', function() {
        if (!this.value)
            $('#%(id)s').val('');
    });
    $('#%(id)s_cl').click(function(){
        $('#%(id)s, #%(id)s_ac').val('');
        return false;
    });
});
</script>
''' % {'id': attrs.get('id', name), 'name': name, 'source': self._source()}

        return mark_safe(hidden + ac + js)

    def _source(self):
        if self.url:
            if hasattr(self.url, '__call__'):
                return '\'%s\'' % self.url()
            return '\'%s\'' % self.url
        else:
            data = json.dumps([
                {
                    'label': force_text(o),
                    'value': o.id,
                } for o in self.queryset.all()
            ])

            return '''function (request, response) {
    var data = %(data)s, ret = [], term = request.term.toLowerCase();
    for (var i=0; i<data.length; ++i) {
        if (data[i].label.toLowerCase().indexOf(term) != -1)
            ret.push(data[i]);
    }
    response(ret);
}
''' % {'data': data}


class InvalidEntry(object):
    pk = None


class MultipleAutocompletionWidget(forms.TextInput):
    """
    You should probably use harvest chosen instead.
    """
    def __init__(self, attrs=None, queryset=None):
        self.queryset = queryset
        super(MultipleAutocompletionWidget, self).__init__(attrs)

    def _possible(self):
        return dict((force_text(o).lower(), o) for o in self.queryset._clone())

    def render(self, name, value, attrs=None, choices=()):
        if value is None:
            value = []
        final_attrs = self.build_attrs(attrs, name=name, type='text')

        if value:
            value = ', '.join(
                force_text(o) for o in self.queryset.filter(id__in=value))

        js = '''<script type="text/javascript">
$(function() {
    function split( val ) {
        return val.split( /,\s*/ );
    }
    function extractLast( term ) {
        return split( term ).pop();
    }

    $( "#%(id)s" )
        // don't navigate away from the field on tab when selecting an item
        .bind( "keydown", function( event ) {
            if ( event.keyCode === $.ui.keyCode.TAB &&
                    $( this ).data( "autocomplete" ).menu.active ) {
                event.preventDefault();
            }
        })
        .autocomplete({
            source: %(source)s,
            search: function() {
                // custom minLength
                var term = extractLast( this.value );
                if ( term.length < 2 ) {
                    return false;
                }
            },
            focus: function() {
                // prevent value inserted on focus
                return false;
            },
            select: function( event, ui ) {
                var terms = split( this.value );
                // remove the current input
                terms.pop();
                // add the selected item
                terms.push( ui.item.value );
                // add placeholder to get the comma-and-space at the end
                terms.push( "" );
                this.value = terms.join( ", " );
                return false;
            }
    });
});
</script>
''' % {
            'id': final_attrs.get('id', name),
            'name': name,
            'source': self._source(),
        }

        return mark_safe(
            '<textarea%s>%s</textarea>' % (
                flatatt(final_attrs),
                value,
            ) + js
        )

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        if not value:
            return []

        possible = self._possible()
        values = [
            s for s in
            [s.strip() for s in value.lower().split(',')]
            if s]
        return list(set(possible.get(s, InvalidEntry).pk for s in values))

    def _source(self):
        return '''function(request, response) {
    response($.ui.autocomplete.filter(%(data)s, extractLast(request.term)));
    }''' % {
            'data': json.dumps(
                [force_text(o) for o in self.queryset._clone()]),
        }
